# this module generates profile URLs for evewho

from urllib.parse import urljoin, quote

from . import *

BASE_URL = 'https://evewho.com'


def _build_url(category: str, eve_id: int) -> str:
    """return url to profile page for an eve entity"""
    
    if category == ESI_CATEGORY_ALLIANCE:        
        partial = 'alliance'

    elif category == ESI_CATEGORY_CORPORATION:        
        partial = 'corporation'

    elif category == ESI_CATEGORY_CHARACTER:
        partial = 'character'
    
    else:
        raise NotImplementedError(
            "Not implemented yet for category:" + category
        )
    
    url = urljoin(
        BASE_URL,
        '{}/{}'.format(partial, int(eve_id))
    )
    return url


def alliance_url(eve_id: int) -> str:
    """url for page about given alliance on evewho"""
    return _build_url(ESI_CATEGORY_ALLIANCE, eve_id)

def character_url(eve_id: int) -> str:
    """url for page about given character on evewho"""
    return _build_url(ESI_CATEGORY_CHARACTER, eve_id)

def corporation_url(eve_id: int) -> str:
    """url for page about given corporation on evewho"""
    return _build_url(ESI_CATEGORY_CORPORATION, eve_id)



from django.contrib.auth.models import Group, Permission
from allianceauth.tests.auth_utils import AuthUtils

DEFAULT_AUTH_GROUP = 'Member'
MODULE_PATH = 'allianceauth.services.modules.discord'

TEST_GUILD_ID = 123456789012345678
TEST_USER_ID = 198765432012345678
TEST_USER_NAME = 'Peter Parker'
TEST_MAIN_NAME = 'Spiderman'
TEST_MAIN_ID = 1005


def add_permissions_to_members():
    permission = Permission.objects.get(codename='access_discord')
    members = Group.objects.get_or_create(name=DEFAULT_AUTH_GROUP)[0]
    AuthUtils.add_permissions_to_groups([permission], [members])



"""Load testing Discord services tasks

This script will load test the Discord service tasks.
Note that his will run against your production Auth.
To run this test start a bunch of celery workers and then run this script directly.

This script requires a user with a Discord account setup through Auth. 
Please provide the respective Discord user ID by setting it as environment variable:

export DISCORD_USER_ID="123456789"
"""

import os
import sys

myauth_dir = '/home/erik997/dev/python/aa/allianceauth-dev/myauth'
sys.path.insert(0, myauth_dir)

import django   # noqa: E402

# init and setup django project
os.environ.setdefault("DJANGO_SETTINGS_MODULE", "myauth.settings.local")
django.setup()

from uuid import uuid1  # noqa: E402

from django.contrib.auth.models import User     # noqa: E402
# from allianceauth.services.modules.discord.tasks import update_groups  # noqa: E402

if 'DISCORD_USER_ID' not in os.environ:
    print('Please set DISCORD_USER_ID')
    exit()

DISCORD_USER_ID = os.environ['DISCORD_USER_ID']


def run_many_updates(runs):        
    user = User.objects.get(discord__uid=DISCORD_USER_ID)
    for _ in range(runs):                
        new_nick = f'Testnick {uuid1().hex}'[:32]
        user.profile.main_character.character_name = new_nick
        user.profile.main_character.save()
        # update_groups.delay(user_pk=user.pk)
        

if __name__ == "__main__":
    run_many_updates(20)



from unittest.mock import Mock, patch

from django.test import TestCase

from ..models import EveCharacter, EveCorporationInfo, \
    EveAllianceInfo, _eve_entity_image_url
from ..providers import Alliance, Corporation, Character


class EveUniverseImageUrlTestCase(TestCase):
    """unit test for _eve_entity_image_url()"""
    
    def test_sizes(self):        
        self.assertEqual(
            _eve_entity_image_url('character', 42),  
            'https://images.evetech.net/characters/42/portrait?size=32'
        )
        self.assertEqual(
            _eve_entity_image_url('character', 42, size=32),  
            'https://images.evetech.net/characters/42/portrait?size=32'
        )
        self.assertEqual(
            _eve_entity_image_url('character', 42, size=64),  
            'https://images.evetech.net/characters/42/portrait?size=64'
        )
        self.assertEqual(
            _eve_entity_image_url('character', 42, size=128),  
            'https://images.evetech.net/characters/42/portrait?size=128'
        )
        self.assertEqual(
            _eve_entity_image_url('character', 42, size=256),  
            'https://images.evetech.net/characters/42/portrait?size=256'
        )
        self.assertEqual(
            _eve_entity_image_url('character', 42, size=512),  
            'https://images.evetech.net/characters/42/portrait?size=512'
        )
        self.assertEqual(
            _eve_entity_image_url('character', 42, size=1024),  
            'https://images.evetech.net/characters/42/portrait?size=1024'
        )
        with self.assertRaises(ValueError):
            _eve_entity_image_url('corporation', 42, size=-5)

        with self.assertRaises(ValueError):
            _eve_entity_image_url('corporation', 42, size=0)
        
        with self.assertRaises(ValueError):
            _eve_entity_image_url('corporation', 42, size=31)
        
        with self.assertRaises(ValueError):
            _eve_entity_image_url('corporation', 42, size=1025)

        with self.assertRaises(ValueError):
            _eve_entity_image_url('corporation', 42, size=2048)


    def test_variant(self):
        self.assertEqual(
            _eve_entity_image_url('character', 42, variant='portrait'),  
            'https://images.evetech.net/characters/42/portrait?size=32'
        )
        self.assertEqual(
            _eve_entity_image_url('alliance', 42, variant='logo'),  
            'https://images.evetech.net/alliances/42/logo?size=32'
        )
        with self.assertRaises(ValueError):
            _eve_entity_image_url('character', 42, variant='logo')
    

    def test_alliance(self):

        self.assertEqual(
            _eve_entity_image_url('alliance', 42),  
            'https://images.evetech.net/alliances/42/logo?size=32'
        )
        self.assertEqual(
            _eve_entity_image_url('corporation', 42),  
            'https://images.evetech.net/corporations/42/logo?size=32'
        )
        self.assertEqual(
            _eve_entity_image_url('character', 42),  
            'https://images.evetech.net/characters/42/portrait?size=32'
        )
        with self.assertRaises(ValueError):
            _eve_entity_image_url('station', 42)


    def test_tenants(self):
        self.assertEqual(
            _eve_entity_image_url('character', 42, tenant='tranquility'),  
            'https://images.evetech.net/characters/42/portrait?size=32&tenant=tranquility'
        )
        self.assertEqual(
            _eve_entity_image_url('character', 42, tenant='singularity'),  
            'https://images.evetech.net/characters/42/portrait?size=32&tenant=singularity'
        )
        with self.assertRaises(ValueError):
            _eve_entity_image_url('character', 42, tenant='xxx')
        

class EveCharacterTestCase(TestCase):
    def test_corporation_prop(self):
        """
        Test that the correct corporation is returned by the corporation property
        """
        character = EveCharacter.objects.create(
            character_id='1234',
            character_name='character.name',
            corporation_id='2345',
            corporation_name='character.corp.name',
            corporation_ticker='character.corp.ticker',
            alliance_id='character.alliance.id',
            alliance_name='character.alliance.name',
        )

        expected = EveCorporationInfo.objects.create(
            corporation_id='2345',
            corporation_name='corp.name',
            corporation_ticker='corp.ticker',
            member_count=10,
            alliance=None,
        )

        incorrect = EveCorporationInfo.objects.create(
            corporation_id='9999',
            corporation_name='corp.name1',
            corporation_ticker='corp.ticker1',
            member_count=10,
            alliance=None,
        )

        self.assertEqual(character.corporation, expected)
        self.assertNotEqual(character.corporation, incorrect)

    def test_corporation_prop_exception(self):
        """
        Check that an exception is raised when the expected
        object is not in the database
        """
        character = EveCharacter.objects.create(
            character_id='1234',
            character_name='character.name',
            corporation_id='2345',
            corporation_name='character.corp.name',
            corporation_ticker='character.corp.ticker',
            alliance_id='character.alliance.id',
            alliance_name='character.alliance.name',
        )

        with self.assertRaises(EveCorporationInfo.DoesNotExist):
            result = character.corporation

    def test_alliance_prop(self):
        """
        Test that the correct alliance is returned by the alliance property
        """
        character = EveCharacter.objects.create(
            character_id='1234',
            character_name='character.name',
            corporation_id='2345',
            corporation_name='character.corp.name',
            corporation_ticker='character.corp.ticker',
            alliance_id='3456',
            alliance_name='character.alliance.name',
        )

        expected = EveAllianceInfo.objects.create(
            alliance_id='3456',
            alliance_name='alliance.name',
            alliance_ticker='alliance.ticker',
            executor_corp_id='alliance.executor_corp_id',
        )

        incorrect = EveAllianceInfo.objects.create(
            alliance_id='9001',
            alliance_name='alliance.name1',
            alliance_ticker='alliance.ticker1',
            executor_corp_id='alliance.executor_corp_id1',
        )

        self.assertEqual(character.alliance, expected)
        self.assertNotEqual(character.alliance, incorrect)

    def test_alliance_prop_exception(self):
        """
        Check that an exception is raised when the expected
        object is not in the database
        """
        character = EveCharacter.objects.create(
            character_id='1234',
            character_name='character.name',
            corporation_id='2345',
            corporation_name='character.corp.name',
            corporation_ticker='character.corp.ticker',
            alliance_id='3456',
            alliance_name='character.alliance.name',
        )

        with self.assertRaises(EveAllianceInfo.DoesNotExist):
            result = character.alliance

    def test_alliance_prop_none(self):
        """
        Check that None is returned when the character has no alliance
        """
        character = EveCharacter.objects.create(
            character_id='1234',
            character_name='character.name',
            corporation_id='2345',
            corporation_name='character.corp.name',
            corporation_ticker='character.corp.ticker',
            alliance_id=None,
            alliance_name=None,
        )

        self.assertIsNone(character.alliance)

    @patch('allianceauth.eveonline.providers.provider')
    def test_update_character(self, mock_provider):
        mock_provider.get_corp.return_value = Corporation(
            id=2002,
            name='Dummy Corp 2',
            ticker='DC2',
            ceo_id=1001,
            members=34,            
        )
        
        my_character = EveCharacter.objects.create(
            character_id='1001',
            character_name='Bruce Wayne',
            corporation_id='2001',
            corporation_name='Dummy Corp 1',
            corporation_ticker='DC1',
            alliance_id='3001',
            alliance_name='Dummy Alliance 1',
        )        
        my_updated_character = Character(
            name='Bruce X. Wayne',
            corp_id=2002
        )
        my_character.update_character(my_updated_character)
        self.assertEqual(my_character.character_name, 'Bruce X. Wayne')

        # todo: add test cases not yet covered, e.g. with alliance


    def test_image_url(self):
        self.assertEqual(
            EveCharacter.generic_portrait_url(42),
            _eve_entity_image_url('character', 42)
        )
        self.assertEqual(
            EveCharacter.generic_portrait_url(42, 256),
            _eve_entity_image_url('character', 42, 256)
        )

    def test_portrait_urls(self):
        x = EveCharacter(
            character_id='42',
            character_name='character.name',
            corporation_id='123',
            corporation_name='corporation.name',            
            corporation_ticker='ABC',            
        )
        self.assertEqual(
            x.portrait_url(),
            _eve_entity_image_url('character', 42)
        )
        self.assertEqual(
            x.portrait_url(64),
            _eve_entity_image_url('character', 42, size=64)
        )       
        self.assertEqual(
            x.portrait_url_32,
            _eve_entity_image_url('character', 42, size=32)
        )
        self.assertEqual(
            x.portrait_url_64,
            _eve_entity_image_url('character', 42, size=64)
        )
        self.assertEqual(
            x.portrait_url_128,
            _eve_entity_image_url('character', 42, size=128)
        )
        self.assertEqual(
            x.portrait_url_256,
            _eve_entity_image_url('character', 42, size=256)
        )


    def test_corporation_logo_urls(self):
        x = EveCharacter(
            character_id='42',
            character_name='character.name',
            corporation_id='123',
            corporation_name='corporation.name',            
            corporation_ticker='ABC',            
        )
        self.assertEqual(
            x.corporation_logo_url(),
            _eve_entity_image_url('corporation', 123)
        )
        self.assertEqual(
            x.corporation_logo_url(256),
            _eve_entity_image_url('corporation', 123, size=256)
        )
        self.assertEqual(
            x.corporation_logo_url_32,
            _eve_entity_image_url('corporation', 123, size=32)
        )
        self.assertEqual(
            x.corporation_logo_url_64,
            _eve_entity_image_url('corporation', 123, size=64)
        )
        self.assertEqual(
            x.corporation_logo_url_128,
            _eve_entity_image_url('corporation', 123, size=128)
        )
        self.assertEqual(
            x.corporation_logo_url_256,
            _eve_entity_image_url('corporation', 123, size=256)
        )


    def test_alliance_logo_urls(self):
        x = EveCharacter(
            character_id='42',
            character_name='character.name',
            corporation_id='123',
            corporation_name='corporation.name',            
            corporation_ticker='ABC',            
        )        
        self.assertEqual(
            x.alliance_logo_url(),
            ''
        )
        self.assertEqual(
            x.alliance_logo_url_32,
            ''
        )
        self.assertEqual(
            x.alliance_logo_url_64,
            ''
        )
        self.assertEqual(
            x.alliance_logo_url_128,
            ''
        )
        self.assertEqual(
            x.alliance_logo_url_256,
            ''
        )
        x.alliance_id = 987
        self.assertEqual(
            x.alliance_logo_url(),
            _eve_entity_image_url('alliance', 987)
        )
        self.assertEqual(
            x.alliance_logo_url(128),
            _eve_entity_image_url('alliance', 987, size=128)
        )
        self.assertEqual(
            x.alliance_logo_url_32,
            _eve_entity_image_url('alliance', 987, size=32)
        )
        self.assertEqual(
            x.alliance_logo_url_64,
            _eve_entity_image_url('alliance', 987, size=64)
        )
        self.assertEqual(
            x.alliance_logo_url_128,
            _eve_entity_image_url('alliance', 987, size=128)
        )
        self.assertEqual(
            x.alliance_logo_url_256,
            _eve_entity_image_url('alliance', 987, size=256)
        )


class EveAllianceTestCase(TestCase):
            
    def test_str(self):
        my_alliance = EveAllianceInfo(
            alliance_id=3001,
            alliance_name='Dummy Alliance 1',
            alliance_ticker='DA1',
            executor_corp_id=2001
        )
        self.assertEqual(str(my_alliance), 'Dummy Alliance 1')
    
    @patch(
        'allianceauth.eveonline.models.EveCorporationInfo.objects.create_corporation'
    )
    def test_populate_alliance(self, mock_create_corporation):
        
        def create_corp(corp_id):
            if corp_id == 2002:
                EveCorporationInfo.objects.create(
                    corporation_id=2002,
                    corporation_name='Dummy Corporation 2',
                    corporation_ticker='DC2',
                    member_count=87,
                )
            else:
                raise ValueError()
        
        mock_EveAllianceProviderManager = Mock()
        mock_EveAllianceProviderManager.get_alliance.return_value = \
            Alliance(
                id=3001, 
                name='Dummy Alliance 1',
                corp_ids=[2001, 2002]
            )
        mock_create_corporation.side_effect = create_corp
        
        EveCorporationInfo.objects.create(
            corporation_id=2001,
            corporation_name='Dummy Corporation 1',
            corporation_ticker='DC1',
            member_count=42,
        )
                
        my_alliance = EveAllianceInfo(
            alliance_id=3001,
            alliance_name='Dummy Alliance 1',
            alliance_ticker='DA1',
            executor_corp_id=2001
        )
        my_alliance.provider = mock_EveAllianceProviderManager
        my_alliance.save()
        my_alliance.populate_alliance()        

        for corporation in EveCorporationInfo.objects\
            .filter(corporation_id__in=[2001, 2002]
        ):
            self.assertEqual(corporation.alliance, my_alliance)
        
    def test_update_alliance_with_object(self):
        my_alliance = EveAllianceInfo.objects.create(
            alliance_id=3001,
            alliance_name='Dummy Alliance 1',
            alliance_ticker='DA1',
            executor_corp_id=2001
        )
        updated_alliance = Alliance(
            id=3002, 
            name='Dummy Alliance 2',
            corp_ids=[2004],                
            executor_corp_id=2004
        )
        my_alliance.update_alliance(updated_alliance)
        my_alliance.refresh_from_db()
        self.assertEqual(int(my_alliance.executor_corp_id), 2004)

        # potential bug
        # update_alliance() is only updateting executor_corp_id when object is given


    def test_update_alliance_wo_object(self):
        mock_EveAllianceProviderManager = Mock()
        mock_EveAllianceProviderManager.get_alliance.return_value = \
            Alliance(
                id=3002, 
                name='Dummy Alliance 2',
                corp_ids=[2004],
                executor_corp_id=2004
            )

        my_alliance = EveAllianceInfo.objects.create(
            alliance_id=3001,
            alliance_name='Dummy Alliance 1',
            alliance_ticker='DA1',
            executor_corp_id=2001
        )
        my_alliance.provider = mock_EveAllianceProviderManager
        my_alliance.save()
        updated_alliance = Alliance(                
                name='Dummy Alliance 2',
                corp_ids=[2004],                
                executor_corp_id=2004
            )
        my_alliance.update_alliance()
        my_alliance.refresh_from_db()
        self.assertEqual(int(my_alliance.executor_corp_id), 2004)
        
        # potential bug
        # update_alliance() is only updateting executor_corp_id nothing else ???


    def test_image_url(self):
        self.assertEqual(
            EveAllianceInfo.generic_logo_url(42),
            _eve_entity_image_url('alliance', 42)
        )
        self.assertEqual(
            EveAllianceInfo.generic_logo_url(42, 256),
            _eve_entity_image_url('alliance', 42, 256)
        )

    def test_logo_url(self):
        x = EveAllianceInfo(
            alliance_id='42',
            alliance_name='alliance.name',
            alliance_ticker='ABC',
            executor_corp_id='123'
        )
        self.assertEqual(
            x.logo_url(),
            'https://images.evetech.net/alliances/42/logo?size=32'
        )
        self.assertEqual(
            x.logo_url(64),
            'https://images.evetech.net/alliances/42/logo?size=64'
        )        
        self.assertEqual(
            x.logo_url_32,
            'https://images.evetech.net/alliances/42/logo?size=32'
        )
        self.assertEqual(
            x.logo_url_64,
            'https://images.evetech.net/alliances/42/logo?size=64'
        )
        self.assertEqual(
            x.logo_url_128,
            'https://images.evetech.net/alliances/42/logo?size=128'
        )
        self.assertEqual(
            x.logo_url_256,
            'https://images.evetech.net/alliances/42/logo?size=256'
        )
        

class EveCorporationTestCase(TestCase):
    
    def setUp(self):
        my_alliance = EveAllianceInfo.objects.create(
            alliance_id=3001,
            alliance_name='Dummy Alliance 1',
            alliance_ticker='DA1',
            executor_corp_id=2001
        )
        self.my_corp = EveCorporationInfo(
            corporation_id=2001,
            corporation_name='Dummy Corporation 1',
            corporation_ticker='DC1',
            member_count=42,
            alliance=my_alliance
        )        

    def test_str(self):        
        self.assertEqual(str(self.my_corp), 'Dummy Corporation 1')

    def test_update_corporation_from_object_w_alliance(self):
        updated_corp = Corporation(
            members=87
        )
        self.my_corp.update_corporation(updated_corp)
        self.assertEqual(self.my_corp.member_count, 87)

        # potential bug
        # update_corporation updates member_count only
    
    def test_update_corporation_no_object_w_alliance(self):
        mock_provider = Mock()
        mock_provider.get_corporation.return_value =  Corporation(
            members=87
        )
        self.my_corp.provider = mock_provider
        
        self.my_corp.update_corporation()
        self.assertEqual(self.my_corp.member_count, 87)
            
    def test_update_corporation_from_object_wo_alliance(self):
        my_corp2 = EveCorporationInfo(
            corporation_id=2011,
            corporation_name='Dummy Corporation 11',
            corporation_ticker='DC11',
            member_count=6
        )
        updated_corp = Corporation(
            members=8
        )
        my_corp2.update_corporation(updated_corp)
        self.assertEqual(my_corp2.member_count, 8)
        self.assertIsNone(my_corp2.alliance)


    def test_image_url(self):
        self.assertEqual(
            EveCorporationInfo.generic_logo_url(42),
            _eve_entity_image_url('corporation', 42)
        )
        self.assertEqual(
            EveCorporationInfo.generic_logo_url(42, 256),
            _eve_entity_image_url('corporation', 42, 256)
        )

    def test_logo_url(self):        
        self.assertEqual(
            self.my_corp.logo_url(),
            'https://images.evetech.net/corporations/2001/logo?size=32'
        )
        self.assertEqual(
            self.my_corp.logo_url(64),
            'https://images.evetech.net/corporations/2001/logo?size=64'
        )
        self.assertEqual(
            self.my_corp.logo_url_32,
            'https://images.evetech.net/corporations/2001/logo?size=32'
        )
        self.assertEqual(
            self.my_corp.logo_url_64,
            'https://images.evetech.net/corporations/2001/logo?size=64'
        )
        self.assertEqual(
            self.my_corp.logo_url_128,
            'https://images.evetech.net/corporations/2001/logo?size=128'
        )
        self.assertEqual(
            self.my_corp.logo_url_256,
            'https://images.evetech.net/corporations/2001/logo?size=256'
        )
        



# this module generates profile URLs for dotlan

from urllib.parse import urljoin, quote

from . import *

BASE_URL = 'http://evemaps.dotlan.net'


def _build_url(category: str, name: str) -> str:
    """return url to profile page for an eve entity"""
    
    if category == ESI_CATEGORY_ALLIANCE:        
        partial = 'alliance'

    elif category == ESI_CATEGORY_CORPORATION:        
        partial = 'corp'

    elif category == ESI_CATEGORY_REGION:
        partial = 'map'
    
    elif category == ESI_CATEGORY_SOLARSYSTEM:
        partial = 'system'
    
    else:
        raise NotImplementedError(
            "Not implemented yet for category:" + category
        )
        
    url = urljoin(
        BASE_URL,
        '{}/{}'.format(partial, quote(str(name).replace(" ", "_")))
        
    )
    return url


def alliance_url(name: str) -> str:
    """url for page about given alliance on dotlan"""
    return _build_url(ESI_CATEGORY_ALLIANCE, name)

def corporation_url(name: str) -> str:
    """url for page about given corporation on dotlan"""
    return _build_url(ESI_CATEGORY_CORPORATION, name)

def region_url(name: str) -> str:
    """url for page about given region on dotlan"""
    return _build_url(ESI_CATEGORY_REGION, name)

def solar_system_url(name: str) -> str:
    """url for page about given solar system on dotlan"""
    return _build_url(ESI_CATEGORY_SOLARSYSTEM, name)


from django.test import TestCase

from ...models import EveCharacter, EveCorporationInfo, EveAllianceInfo
from .. import dotlan, zkillboard, evewho
from ...templatetags import evelinks


class TestEveWho(TestCase):

    def test_alliance_url(self):
        self.assertEqual(
            evewho.alliance_url(12345678),
            'https://evewho.com/alliance/12345678'
        )
    
    def test_corporation_url(self):
        self.assertEqual(
            evewho.corporation_url(12345678),
            'https://evewho.com/corporation/12345678'
        )  

    def test_character_url(self):
        self.assertEqual(
            evewho.character_url(12345678),
            'https://evewho.com/character/12345678'
        )


class TestDotlan(TestCase):

    def test_alliance_url(self):
        self.assertEqual(
            dotlan.alliance_url('Wayne Enterprices'),
            'http://evemaps.dotlan.net/alliance/Wayne_Enterprices'
        )

    def test_corporation_url(self):
        self.assertEqual(
            dotlan.corporation_url('Wayne Technology'),
            'http://evemaps.dotlan.net/corp/Wayne_Technology'
        )
        self.assertEqual(
            dotlan.corporation_url('Crédit Agricole'),
            'http://evemaps.dotlan.net/corp/Cr%C3%A9dit_Agricole'
        )

    def test_region_url(self):
        self.assertEqual(
            dotlan.region_url('Black Rise'),
            'http://evemaps.dotlan.net/map/Black_Rise'
        )
    
    def test_solar_system_url(self):
        self.assertEqual(
            dotlan.solar_system_url('Jita'),
            'http://evemaps.dotlan.net/system/Jita'
        )


class TestZkillboard(TestCase):

    def test_alliance_url(self):
        self.assertEqual(
            zkillboard.alliance_url(12345678),
            'https://zkillboard.com/alliance/12345678/'
        )

    def test_corporation_url(self):
        self.assertEqual(
            zkillboard.corporation_url(12345678),
            'https://zkillboard.com/corporation/12345678/'
        )        

    def test_character_url(self):
        self.assertEqual(
            zkillboard.character_url(12345678),
            'https://zkillboard.com/character/12345678/'
        )
    

    def test_region_url(self):
        self.assertEqual(
            zkillboard.region_url(12345678),
            'https://zkillboard.com/region/12345678/'
        )

    def test_solar_system_url(self):
        self.assertEqual(
            zkillboard.solar_system_url(12345678),
            'https://zkillboard.com/system/12345678/'
        )




import logging

from requests.exceptions import HTTPError

from django.contrib.auth.models import User
from django.db import models
from django.utils.translation import gettext_lazy

from allianceauth.notifications import notify

from . import __title__
from .app_settings import DISCORD_GUILD_ID
from .discord_client import DiscordClient, DiscordApiBackoff
from .managers import DiscordUserManager
from .utils import LoggerAddTag


logger = LoggerAddTag(logging.getLogger(__name__), __title__)


class DiscordUser(models.Model):

    USER_RELATED_NAME = 'discord'
    
    user = models.OneToOneField(
        User, 
        primary_key=True, 
        on_delete=models.CASCADE, 
        related_name=USER_RELATED_NAME,
        help_text='Auth user owning this Discord account'
    )
    uid = models.BigIntegerField(
        db_index=True,
        help_text='user\'s ID on Discord'
    )
    username = models.CharField(
        max_length=32, 
        default='', 
        blank=True,
        db_index=True,
        help_text='user\'s username on Discord'
    )
    discriminator = models.CharField(
        max_length=4, 
        default='', 
        blank=True, 
        help_text='user\'s discriminator on Discord'
    )
    activated = models.DateTimeField(
        default=None, 
        null=True, 
        blank=True,
        help_text='Date & time this service account was activated'
    )

    objects = DiscordUserManager()

    class Meta:
        permissions = (
            ("access_discord", "Can access the Discord service"),
        )

    def __str__(self):
        return f'{self.user.username} - {self.uid}'

    def __repr__(self):
        return f'{type(self).__name__}(user=\'{self.user}\', uid={self.uid})'

    def update_nickname(self) -> bool:
        """Update nickname with formatted name of main character
                
        Returns:
        - True on success
        - None if user is no longer a member of the Discord server
        - False on error or raises exception
        """        
        requested_nick = DiscordUser.objects.user_formatted_nick(self.user)
        if requested_nick:            
            client = DiscordUser.objects._bot_client()            
            success = client.modify_guild_member(
                guild_id=DISCORD_GUILD_ID,
                user_id=self.uid,
                nick=requested_nick
            )
            if success:
                logger.info('Nickname for %s has been updated', self.user)
            else:
                logger.warning('Failed to update nickname for %s', self.user)
            return success
         
        else:
            return False

    def update_groups(self) -> bool:
        """update groups for a user based on his current group memberships. 
        Will add or remove roles of a user as needed.
        
        Returns:
        - True on success
        - None if user is no longer a member of the Discord server
        - False on error or raises exception
        """
        role_names = DiscordUser.objects.user_group_names(self.user)        
        client = DiscordUser.objects._bot_client()
        requested_role_ids = self._guild_get_or_create_role_ids(client, role_names)
        logger.debug(
            'Requested to update groups for user %s: %s', self.user, requested_role_ids
        )        
        success = client.modify_guild_member(
            guild_id=DISCORD_GUILD_ID,
            user_id=self.uid,
            role_ids=requested_role_ids
        )
        if success:
            logger.info('Groups for %s have been updated', self.user)
        else:
            logger.warning('Failed to update groups for %s', self.user)
        return success

    def delete_user(
        self, notify_user: bool = False, is_rate_limited: bool = True
    ) -> bool:
        """Deletes the Discount user both on the server and locally

        Params:
        - notify_user: When True will sent a notification to the user 
        informing him about the deleting of his account
        - is_rate_limited: When False will disable default rate limiting (use with care)
        
        Returns True when successful, otherwise False or raises exceptions
        Return None if user does no longer exist
        """
        try:
            client = DiscordUser.objects._bot_client(is_rate_limited=is_rate_limited)
            success = client.remove_guild_member(
                guild_id=DISCORD_GUILD_ID, user_id=self.uid
            )
            if success is not False:
                deleted_count, _ = self.delete()
                if deleted_count > 0:
                    if notify_user:
                        notify(
                            user=self.user, 
                            title=gettext_lazy('Discord Account Disabled'), 
                            message=gettext_lazy(
                                'Your Discord account was disabeled automatically '
                                'by Auth. If you think this was a mistake, '
                                'please contact an admin.'
                            ),
                            level='warning'
                        )
                    logger.info('Account for user %s was deleted.', self.user)
                    return True
                else:
                    logger.debug('Account for user %s was already deleted.', self.user)
                    return None
            
            else:
                logger.warning(
                    'Failed to remove user %s from the Discord server', self.user
                )
                return False
        
        except (HTTPError, ConnectionError, DiscordApiBackoff) as ex:
            logger.exception(
                'Failed to remove user %s from Discord server: %s', self.user, ex
            )
            return False        

    @staticmethod
    def _guild_get_or_create_role_ids(client: DiscordClient, role_names: list) -> list:
        """wrapper for DiscordClient.match_guild_roles_to_names()
        that only returns the list of IDs
        """
        return [
            x[0]['id'] for x in client.match_guild_roles_to_names(
                guild_id=DISCORD_GUILD_ID, role_names=role_names
            )
        ]



from django_webtest import WebTest
from unittest.mock import patch

from django.shortcuts import reverse

from allianceauth.tests.auth_utils import AuthUtils

from . import (
    add_permissions_to_members, 
    MODULE_PATH, 
    TEST_USER_NAME,    
    TEST_MAIN_NAME,
    TEST_MAIN_ID
)


class TestServiceUserActivation(WebTest):
    
    def setUp(self):
        self.member = AuthUtils.create_member(TEST_USER_NAME)
        AuthUtils.add_main_character_2(
            self.member, 
            TEST_MAIN_NAME, 
            TEST_MAIN_ID,
            disconnect_signals=True
        )
        add_permissions_to_members()
    
    @patch(MODULE_PATH + '.views.messages')
    @patch(MODULE_PATH + '.models.DiscordUser.objects.add_user')
    @patch(MODULE_PATH + '.managers.OAuth2Session')
    def test_user_activation(
        self, mock_OAuth2Session, mock_add_user, mock_messages
    ): 
        authentication_code = 'auth_code'
        mock_add_user.return_value = True
        oauth_url = 'https://www.example.com/oauth'
        state = ''
        mock_OAuth2Session.return_value.authorization_url.return_value = \
            oauth_url, state
        
        # login
        self.app.set_user(self.member)
        
        # click activate on the service page
        response = self.app.get(reverse('discord:activate'))
        
        # check we got a redirect to Discord OAuth        
        self.assertRedirects(
            response, expected_url=oauth_url, fetch_redirect_response=False
        )

        # simulate Discord callback
        response = self.app.get(
            reverse('discord:callback'), params={'code': authentication_code}
        )

        # user was added to Discord
        self.assertTrue(mock_add_user.called)

        # user got a success message
        self.assertTrue(mock_messages.success.called)



from unittest.mock import patch, Mock

from requests.exceptions import HTTPError

from django.test import TestCase

from allianceauth.tests.auth_utils import AuthUtils

from . import TEST_USER_NAME, TEST_USER_ID, TEST_MAIN_NAME, TEST_MAIN_ID, MODULE_PATH
from ..discord_client import DiscordClient, DiscordApiBackoff
from ..models import DiscordUser
from ..utils import set_logger_to_file


logger = set_logger_to_file(MODULE_PATH + '.models', __file__)


class TestBasicsAndHelpers(TestCase):

    def test_str(self):
        user = AuthUtils.create_user(TEST_USER_NAME)
        discord_user = DiscordUser.objects.create(user=user, uid=TEST_USER_ID)
        expected = 'Peter Parker - 198765432012345678'
        self.assertEqual(str(discord_user), expected)

    def test_repr(self):
        user = AuthUtils.create_user(TEST_USER_NAME)
        discord_user = DiscordUser.objects.create(user=user, uid=TEST_USER_ID)
        expected = 'DiscordUser(user=\'Peter Parker\', uid=198765432012345678)'
        self.assertEqual(repr(discord_user), expected)
    
    def test_guild_get_or_create_role_ids(self):
        mock_client = Mock(spec=DiscordClient)
        mock_client.match_guild_roles_to_names.return_value = \
            [({'id': 1, 'name': 'alpha'}, True), ({'id': 2, 'name': 'bravo'}, True)]
        
        result = DiscordUser._guild_get_or_create_role_ids(mock_client, [])
        excepted = [1, 2]
        self.assertEqual(set(result), set(excepted))


@patch(MODULE_PATH + '.managers.DiscordClient', spec=DiscordClient)
class TestUpdateNick(TestCase):

    def setUp(self): 
        self.user = AuthUtils.create_user(TEST_USER_NAME)
        self.discord_user = DiscordUser.objects.create(
            user=self.user, uid=TEST_USER_ID
        )

    @staticmethod
    def user_info(nick):
        return {
            'user': {
                'id': TEST_USER_ID,
                'username': TEST_USER_NAME
            },
            'nick': nick,
            'roles': [1, 2, 3]
        }
   
    def test_can_update(self, mock_DiscordClient):
        AuthUtils.add_main_character_2(self.user, TEST_MAIN_NAME, TEST_MAIN_ID)        
        mock_DiscordClient.return_value.modify_guild_member.return_value = True
        
        result = self.discord_user.update_nickname()
        self.assertTrue(result)
        self.assertTrue(mock_DiscordClient.return_value.modify_guild_member.called)

    def test_dont_update_if_user_has_no_main(self, mock_DiscordClient):        
        mock_DiscordClient.return_value.modify_guild_member.return_value = False
        
        result = self.discord_user.update_nickname()
        self.assertFalse(result)
        self.assertFalse(mock_DiscordClient.return_value.modify_guild_member.called)
    
    def test_return_none_if_user_no_longer_a_member(
        self, mock_DiscordClient
    ):
        AuthUtils.add_main_character_2(self.user, TEST_MAIN_NAME, TEST_MAIN_ID)
        mock_DiscordClient.return_value.modify_guild_member.return_value = None
        
        result = self.discord_user.update_nickname()
        self.assertIsNone(result)
        self.assertTrue(mock_DiscordClient.return_value.modify_guild_member.called)        

    def test_return_false_if_api_returns_false(self, mock_DiscordClient):
        AuthUtils.add_main_character_2(self.user, TEST_MAIN_NAME, TEST_MAIN_ID)        
        mock_DiscordClient.return_value.modify_guild_member.return_value = False
        
        result = self.discord_user.update_nickname()
        self.assertFalse(result)
        self.assertTrue(mock_DiscordClient.return_value.modify_guild_member.called)


@patch(MODULE_PATH + '.models.notify')
@patch(MODULE_PATH + '.managers.DiscordClient', spec=DiscordClient)
class TestDeleteUser(TestCase):

    def setUp(self): 
        self.user = AuthUtils.create_user(TEST_USER_NAME)        
        self.discord_user = DiscordUser.objects.create(
            user=self.user, uid=TEST_USER_ID
        )

    def test_can_delete_user(self, mock_DiscordClient, mock_notify):        
        mock_DiscordClient.return_value.remove_guild_member.return_value = True
        result = self.discord_user.delete_user()
        self.assertTrue(result)
        self.assertFalse(
            DiscordUser.objects.filter(user=self.user, uid=TEST_USER_ID).exists()
        )
        self.assertTrue(mock_DiscordClient.return_value.remove_guild_member.called)
        self.assertFalse(mock_notify.called)

    def test_can_delete_user_and_notify_user(self, mock_DiscordClient, mock_notify):
        mock_DiscordClient.return_value.remove_guild_member.return_value = True
        result = self.discord_user.delete_user(notify_user=True)
        self.assertTrue(result)
        self.assertTrue(mock_notify.called)

    def test_can_delete_user_when_member_is_unknown(
        self, mock_DiscordClient, mock_notify
    ): 
        mock_DiscordClient.return_value.remove_guild_member.return_value = None
        result = self.discord_user.delete_user()
        self.assertTrue(result)
        self.assertFalse(
            DiscordUser.objects.filter(user=self.user, uid=TEST_USER_ID).exists()
        )
        self.assertTrue(mock_DiscordClient.return_value.remove_guild_member.called)
        self.assertFalse(mock_notify.called)

    def test_return_false_when_api_fails(self, mock_DiscordClient, mock_notify):
        mock_DiscordClient.return_value.remove_guild_member.return_value = False
        result = self.discord_user.delete_user()
        self.assertFalse(result)

    def test_dont_notify_if_user_was_already_deleted_and_return_none(
        self, mock_DiscordClient, mock_notify
    ):
        mock_DiscordClient.return_value.remove_guild_member.return_value = None
        DiscordUser.objects.get(pk=self.discord_user.pk).delete()
        result = self.discord_user.delete_user()
        self.assertIsNone(result)
        self.assertFalse(
            DiscordUser.objects.filter(user=self.user, uid=TEST_USER_ID).exists()
        )
        self.assertTrue(mock_DiscordClient.return_value.remove_guild_member.called)
        self.assertFalse(mock_notify.called)

    def test_return_false_on_api_backoff(self, mock_DiscordClient, mock_notify):
        mock_DiscordClient.return_value.remove_guild_member.side_effect = \
            DiscordApiBackoff(999)
        result = self.discord_user.delete_user()
        self.assertFalse(result)

    def test_return_false_on_http_error(self, mock_DiscordClient, mock_notify):
        mock_exception = HTTPError('error')
        mock_exception.response = Mock()
        mock_exception.response.status_code = 500
        mock_DiscordClient.return_value.remove_guild_member.side_effect = \
            mock_exception
        result = self.discord_user.delete_user()
        self.assertFalse(result)


@patch(MODULE_PATH + '.managers.DiscordClient', spec=DiscordClient)
@patch(MODULE_PATH + '.models.DiscordUser._guild_get_or_create_role_ids')
@patch(MODULE_PATH + '.models.DiscordUser.objects.user_group_names')
class TestUpdateGroups(TestCase):

    def setUp(self): 
        self.user = AuthUtils.create_user(TEST_USER_NAME)
        self.discord_user = DiscordUser.objects.create(
            user=self.user, uid=TEST_USER_ID
        )
        
    def test_can_update(
        self, 
        mock_user_group_names, 
        mock_guild_get_or_create_role_ids, 
        mock_DiscordClient
    ):        
        roles_requested = [1, 2, 3]
        mock_user_group_names.return_value = []
        mock_guild_get_or_create_role_ids.return_value = roles_requested
        mock_DiscordClient.return_value.modify_guild_member.return_value = True
        
        result = self.discord_user.update_groups()
        self.assertTrue(result)
        self.assertTrue(mock_DiscordClient.return_value.modify_guild_member.called)
    
    def test_return_none_if_user_no_longer_a_member(
        self,
        mock_user_group_names, 
        mock_guild_get_or_create_role_ids, 
        mock_DiscordClient
    ):        
        roles_requested = [1, 2, 3]
        mock_user_group_names.return_value = []
        mock_guild_get_or_create_role_ids.return_value = roles_requested
        mock_DiscordClient.return_value.modify_guild_member.return_value = None
                
        result = self.discord_user.update_groups()
        self.assertIsNone(result)
        self.assertTrue(mock_DiscordClient.return_value.modify_guild_member.called)

    def test_return_false_if_api_returns_false(
        self, 
        mock_user_group_names, 
        mock_guild_get_or_create_role_ids, 
        mock_DiscordClient
    ):
        roles_requested = [1, 2, 3]
        mock_user_group_names.return_value = []
        mock_guild_get_or_create_role_ids.return_value = roles_requested
        mock_DiscordClient.return_value.modify_guild_member.return_value = False      
        
        result = self.discord_user.update_groups()
        self.assertFalse(result)
        self.assertTrue(mock_DiscordClient.return_value.modify_guild_member.called)
